@@ -50,7 +50,9 @@ except ImportError:
         def __getattr__(self, attr):
             return pbs.Command(attr)
     sh = Sh()
-from .gitwrapperlibexceptions import ExecutableNotFound
+from .gitwrapperlibexceptions import (ExecutableNotFound,
+                                      BranchNotFound,
+                                      RemoteOriginError)
 
 __author__ = '''Costas Tyfoxylos <costas.tyf@gmail.com>'''
 __docformat__ = '''google'''
@@ -123,21 +125,23 @@ class Git:
         """Adds the remote origin."""
         self._git.remote('add', 'origin', url)
 
-    def push_master(self):
-        """Pushes to master."""
-        self._git.push('origin', 'master')
+    def push_default(self):
+        """Pushes to the default branch."""
+        branch = self.get_default_branch()
+        self._git.push('origin', branch)
 
-    def push_force_master(self):
-        """Pushes to master."""
-        self._git.push('origin', 'master', '--force')
-
-    def push_force_branch(self, branch):
-        """Pushes to master."""
+    def push_force_default(self):
+        """Force pushes to the default branch."""
+        branch = self.get_default_branch()
         self._git.push('origin', branch, '--force')
 
-    def branch_upstream_to_master(self):
-        """Branches upstream to master."""
-        self._git.branch('-u', 'origin/master')
+    def push_branch(self, branch):
+        """Pushes to a branch."""
+        self._git.push('origin', branch)
+
+    def push_force_branch(self, branch):
+        """Force pushes to a branch."""
+        self._git.push('origin', branch, '--force')
 
     def get_branches(self):
         """Returns a list of the branches."""
@@ -159,6 +163,20 @@ class Git:
                      if branch.startswith('*')),
                     None)
 
+    def get_default_branch(self):
+        """Returns the remote default branch."""
+        show_origin_text = str(self._git.remote('show', 'origin'))
+        try:
+            branch = re.search(r'HEAD branch: (\S+)', show_origin_text).group(1)
+        except (IndexError, AttributeError):
+            raise RemoteOriginError(f'git remote show origin command did not respond as expected, '
+                                    f'received :{show_origin_text}')
+        if branch == '(unknown)':
+            message = 'Failed to detect default remote branch, please check your remote settings.'
+            self._logger.error(message)
+            raise BranchNotFound(message)
+        return branch
+
     def create_branch(self, name):
         """Creates a branch."""
         self._git.branch(name)
